package com.sxtvs.open.api.news.service.impl;

import co.elastic.clients.elasticsearch.ElasticsearchClient;
import co.elastic.clients.elasticsearch._types.SortOptions;
import co.elastic.clients.elasticsearch._types.SortOrder;
import co.elastic.clients.elasticsearch._types.query_dsl.MultiMatchQuery;
import co.elastic.clients.elasticsearch._types.query_dsl.Query;
import co.elastic.clients.elasticsearch._types.query_dsl.RangeQuery;
import co.elastic.clients.elasticsearch.core.SearchRequest;
import co.elastic.clients.elasticsearch.core.SearchResponse;
import co.elastic.clients.json.JsonData;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.sxtvs.open.api.news.dto.DataRequestDTO;
import com.sxtvs.open.api.news.entity.YoumeiData;
import com.sxtvs.open.api.news.entity.YoumeiEsData;
import com.sxtvs.open.api.news.mapper.YoumeiDataMapper;
import com.sxtvs.open.api.news.service.IYoumeiDataService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import jakarta.annotation.Resource;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author syj
 * @since 2023-02-20
 */
@Service
public class YoumeiDataServiceImpl extends ServiceImpl<YoumeiDataMapper, YoumeiData> implements IYoumeiDataService {
    @Resource
    private ElasticsearchClient elasticsearchClient;

    @Resource
    private YoumeiOffsetServiceImpl youmeiOffsetService;


    public Page<YoumeiEsData> search(DataRequestDTO dataRequestDTO){
        SearchResponse<YoumeiData> response = null;
        Page<YoumeiEsData> page = new Page<>(dataRequestDTO.getPage(), dataRequestDTO.getPageSize());
        int offset = (int) ((page.getCurrent() - 1) * page.getSize());
        DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        Long maxId = youmeiOffsetService.getById(4).getOffset();
        try {
            SearchRequest.Builder builder = new SearchRequest.Builder();
            builder.index("news_data").from(offset).size((int) page.getSize());
            if(dataRequestDTO.getStart() != null){

                String start = dataRequestDTO.getStart().format(dateTimeFormatter);
                String end = dataRequestDTO.getEnd().format(dateTimeFormatter);
                builder.query(Query.of(x->x.range(y -> y.field("publishTime").gte(JsonData.of(start)).lte(JsonData.of(end)))));
            }
            if("".equals(dataRequestDTO.getKeywords())){
                builder.sort(SortOptions.of(
                        so -> so.field(fs->fs.field("offset").order(SortOrder.Desc))
                ));
            }else{
                builder.query(
                        Query.of(y -> y.multiMatch(MultiMatchQuery.of(z -> z.fields(Arrays.asList("title", "content")).query(dataRequestDTO.getKeywords()))))
                ).minScore(10D);
            }
            builder.postFilter(Query.of(x->x.range(y->y.field("offset").gte(JsonData.of(maxId - 10000)))));
            SearchRequest searchRequest = builder.build();

            response = elasticsearchClient.search(searchRequest, YoumeiData.class);
        } catch (IOException e) {
            e.printStackTrace();
        }
        List<YoumeiEsData> data = new ArrayList<>();
        long total = 0;
        if (response != null) {
            response.hits().hits().forEach(x -> {
                assert x.source() != null;
                data.add(YoumeiEsData.of(x.source()));
            });
            if (response.hits().total() != null) {
                total = response.hits().total().value();
            }
        }
        page.setTotal(total);
        page.setRecords(data);

        return page;
    }
}
